"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const index_1 = require("../index");
const log_1 = require("../util/log");
async function main(argv) {
    const [, bin, ...args] = argv;
    const name = path_1.basename(bin);
    const argMap = {};
    const files = [];
    for (let i = 0; i < args.length; i++) {
        const arg = args[i];
        if (arg[0] === '-') {
            argMap[arg] = true;
            if (arg === '--concurrent' || arg === '-c'
                || arg === '--loglevel' || arg === '-l') {
                argMap[arg] = args[++i];
            }
        }
        else
            files.push(arg);
    }
    const help = argMap['--help'] || argMap['-h'];
    const version = argMap['--version'] || argMap['-v'];
    const recursive = !!(argMap['--recursive'] || argMap['-r']);
    const debug = argMap['--debug'] || argMap['-d'];
    const quiet = argMap['--quiet'] || argMap['-q'];
    const dryrun = !!(argMap['--dryrun'] || argMap['-D']);
    const concurrent = argMap['--concurrent'] || argMap['-c'];
    const logLevel = argMap['--loglevel'] || argMap['-l'];
    if (help)
        return log_1.raw(helpMessage(name));
    if (version)
        return log_1.raw(require(path_1.resolve(__dirname, '../../package.json')).version);
    if (files.length < 2) {
        throw new Error(`${name} missing file operand, usage:\n\n${helpMessage(name)}`);
    }
    const target = files.pop();
    const url = new URL(target);
    const options = {
        receiver: url.origin, recursive, dryrun, concurrent
    };
    if (debug)
        options['logLevel'] = log_1.LogLevel.DEBUG;
    if (quiet)
        options['logLevel'] = log_1.LogLevel.NONE;
    if (logLevel)
        options['logLevel'] = +logLevel;
    return index_1.fcp(files, url.pathname, options);
}
exports.main = main;
function helpMessage(name) {
    return `${name} <SOURCE..> <TARGET>

Positionals:
  SOURCE..  source file or directory            [array]
  TARGET    target file or directory            [string]

Options:
  --version        Show version number          [boolean]
  --help           Show usage instructions.     [boolean]
  --recursive, -r  push directories recursively [boolean]
  --concurrent, -c max concurrent http request  [number]
  --loglevel, -l   0,1,2,3,4,5,6                [number]
  --quiet, -q      print nothing, equiv -l 6    [boolean]
  --dryrun, -D     print tasks without push     [boolean]
  --debug, -d      debug mode, equiv -l 0       [boolean]

Examples:
  fcp ./a.txt http://example.com:8210/tmp/a.txt  a.txt -> /tmp/a.txt
  fcp -r ./dir http://example.com:8210/tmp/dir   dir -> /tmp/dir
  fcp -r ./dir http://example.com:8210/tmp/dir/  dir -> /tmp/dir/dir
  fcp a.txt b.txt http://example.com:8210/tmp/   {a,b}.txt -> /tmp/{a,b}.txt`;
}
exports.helpMessage = helpMessage;
