import { Options, NormalizedOptions } from './options';
interface Task {
    source: string;
    dest: string;
}
/**
 * 拷贝文件。行为类似 fcp、scp，和 push()、pushMultiple() 的不同在于：
 * 1. source 可以是目录，会递归进去拷贝
 * 2. dest 可以以 / 结尾表示放到目录下面
 *
 * @param sources 本地文件/目录路径列表，基于 cwd 解析
 * @param dest 远程文件路径，为绝对路径
 * @param options 推送参数
 */
export declare function fcp(sources: string | string[], dest: string, raw: Options): Promise<void>;
/**
 * 推送一组文件
 *
 * @param tasks 一组文件推送任务
 * @param options 推送参数
 */
export declare function push(tasks: Task[], raw: Options | NormalizedOptions): Promise<void>;
/**
 * 推送单个文件
 *
 * @param source 本地文件路径，基于 cwd 解析
 * @param dest 远程文件路径，为绝对路径
 * @param options 推送参数
 */
export declare function pushFile(source: string, dest: string, raw: Options | NormalizedOptions): Promise<void>;
export declare function makit(raw?: Options): <T extends {
    target: string;
    dependencyFullPath: () => string;
}>(ctx: T) => Promise<void>;
export {};
