"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const log_1 = require("./util/log");
function normalize(raw, additional = {}) {
    const options = {
        ...additional,
        ...(typeof raw === 'string' ? require(path_1.resolve(process.cwd(), raw)) : raw)
    };
    log_1.debug('options', options, 'parsed from raw: ', raw, 'additional:', additional);
    if (!options.receiver)
        throw new Error('options.receiver is required!');
    return {
        ...options,
        receiver: options.receiver,
        logLevel: defaultTo(options.logLevel, log_1.LogLevel.INFO),
        recursive: defaultTo(options.recursive, false),
        uploadAPI: options.receiver + '/v1/upload',
        authAPI: options.receiver + '/v1/authorize',
        validateAPI: options.receiver + '/v1/validate',
        retry: defaultTo(options.retry, 3),
        concurrent: defaultTo(options.concurrent, options.parallelPushCount, 100),
        normalized: true
    };
}
exports.normalize = normalize;
function defaultTo(...args) {
    for (const arg of args)
        if (arg !== undefined)
            return arg;
}
function isNormalizedOptions(options) {
    return options['normalized'];
}
exports.isNormalizedOptions = isNormalizedOptions;
