"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = require("os");
const json_1 = require("./util/json");
const fs_1 = require("./util/fs");
const HOME = os_1.homedir();
exports.FIS_TOKEN_FILE = `${HOME}/.fis3-tmp/deploy.json`;
exports.FHP_TOKEN_FILE = `${HOME}/.fis-http-push.json`;
let token;
async function getToken() {
    if (token) {
        return token;
    }
    const TOKEN_PATH = await tokenPath();
    try {
        const content = await fs_1.readFile(TOKEN_PATH, 'utf8');
        token = json_1.tryParseJSON(content);
    }
    catch (err) {
        if (err.code === 'ENOENT')
            token = {};
        else
            throw err;
    }
    return token;
}
exports.getToken = getToken;
async function writeToken(options) {
    await fs_1.writeFile(await tokenPath(), JSON.stringify(options, null, 2));
    token = options;
}
exports.writeToken = writeToken;
async function tokenPath() {
    // 如果 FIS3 已经有可用 TOKEN，就先用它
    return (await fs_1.exists(exports.FIS_TOKEN_FILE)) ? exports.FIS_TOKEN_FILE : exports.FHP_TOKEN_FILE;
}
function clear() {
    token = null;
}
exports.clear = clear;
