"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const log_1 = require("../util/log");
class LimitedConcurrent {
    constructor(limit = Infinity) {
        this.limit = limit;
        this.pending = [];
        this.m = this.limit;
    }
    setLimit(newLimit) {
        this.m += newLimit - this.limit;
        this.limit = newLimit;
    }
    queue(fn) {
        return new Promise((resolve, reject) => {
            this.pending.push({ fn, resolve, reject });
            this.signal();
        });
    }
    signal() {
        if (!this.pending.length || !this.m)
            return;
        this.m--;
        const { fn, resolve, reject } = this.pending.shift();
        Promise.resolve().then(() => fn()).then((ret) => {
            log_1.debug('limited concurrent resolving');
            resolve(ret);
            this.m++;
            this.signal();
        }, (err) => {
            log_1.debug('limited concurrent rejecting', err);
            reject(err);
            this.m++;
            this.signal();
        });
    }
}
exports.LimitedConcurrent = LimitedConcurrent;
