"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
/* eslint-disable no-console */
var OutStream;
(function (OutStream) {
    OutStream[OutStream["STDOUT"] = 0] = "STDOUT";
    OutStream[OutStream["STDERR"] = 1] = "STDERR";
})(OutStream = exports.OutStream || (exports.OutStream = {}));
var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["NONE"] = 5] = "NONE";
    LogLevel[LogLevel["ERROR"] = 4] = "ERROR";
    LogLevel[LogLevel["WARN"] = 3] = "WARN";
    LogLevel[LogLevel["INFO"] = 2] = "INFO";
    LogLevel[LogLevel["DEBUG"] = 1] = "DEBUG";
})(LogLevel = exports.LogLevel || (exports.LogLevel = {}));
let logLevel = LogLevel.INFO;
function setLogLevel(level) {
    if (typeof level === 'number')
        logLevel = level;
}
exports.setLogLevel = setLogLevel;
function getLogLevel() {
    return logLevel;
}
exports.getLogLevel = getLogLevel;
/**
 * Log Implementation
 */
let impl = [console.log, console.error];
function setLogImpl(newImpl) {
    impl = newImpl;
}
exports.setLogImpl = setLogImpl;
function getLogImpl() {
    return impl;
}
exports.getLogImpl = getLogImpl;
function restoreLogImpl() {
    impl = [console.log, console.error];
}
exports.restoreLogImpl = restoreLogImpl;
/**
 * Log Interfaces
 */
function error(...args) {
    return doLog(OutStream.STDERR, LogLevel.ERROR, 'red', ...args);
}
exports.error = error;
function warn(...args) {
    return doLog(OutStream.STDERR, LogLevel.WARN, 'yellow', ...args);
}
exports.warn = warn;
function success(...args) {
    return doLog(OutStream.STDOUT, LogLevel.INFO, 'green', ...args);
}
exports.success = success;
function log(...args) {
    return doLog(OutStream.STDOUT, LogLevel.INFO, 'dim', ...args);
}
exports.log = log;
function raw(...args) {
    return impl[0](...args);
}
exports.raw = raw;
function debug(...args) {
    const level = process.env.DEBUG === 'fhp' ? Infinity : LogLevel.DEBUG;
    return doLog(OutStream.STDERR, level, 'none', ...args);
}
exports.debug = debug;
function dateStr(now = new Date()) {
    const m = now.getMonth() + 1;
    const d = now.getDate();
    return (m < 10 ? '0' + m : m) + '-' + (d < 10 ? '0' + d : d) + ' ' + now.toTimeString().substr(0, 8);
}
exports.dateStr = dateStr;
function doLog(out, level, color, ...args) {
    if (level < logLevel)
        return;
    const timeInfo = '[' + dateStr() + ']';
    args.unshift(color === 'none' ? timeInfo : chalk_1.default[color](timeInfo));
    impl[out](...args);
    return args;
}
exports.doLog = doLog;
