"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function wait(time) {
    return new Promise(resolve => setTimeout(resolve, time));
}
exports.wait = wait;
function fromCallback(fn) {
    return new Promise((resolve, reject) => {
        fn((err, result) => (err ? reject(err) : resolve(result)));
    });
}
exports.fromCallback = fromCallback;
function singleton(fn) {
    const queue = [];
    return (...args) => new Promise((resolve, reject) => {
        queue.push([resolve, reject]);
        // 后续的都直接返回，由第一个触发
        if (queue.length > 1)
            return;
        fn(...args).then(ret => {
            while (queue.length) {
                const [resolve] = queue.pop();
                resolve(ret);
            }
        }).catch(err => {
            while (queue.length) {
                const [, reject] = queue.pop();
                reject(err);
            }
        });
    });
}
exports.singleton = singleton;
