"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const json_1 = require("./json");
function postURLEncoded(url, data) {
    const content = Object.keys(data).map(key => `${key}=${encodeURIComponent(data[key])}`).join('&');
    const options = {
        ...optionsFromUrl(url),
        method: 'POST',
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded;charset=utf-8'
        }
    };
    return request(options, [content]);
}
exports.postURLEncoded = postURLEncoded;
function postFormEncoded(url, boundary, data) {
    const length = data.reduce((prev, item) => prev + Buffer.from(item).length, 0);
    const options = {
        ...optionsFromUrl(url),
        method: 'POST',
        headers: {
            'Content-Length': length,
            'Content-Type': 'multipart/form-data; boundary=' + boundary
        }
    };
    return request(options, data);
}
exports.postFormEncoded = postFormEncoded;
function request(options, data) {
    return new Promise((resolve, reject) => {
        const { request } = options.protocol === 'https:' ? require('https') : require('http');
        const req = request(options, responseHandler);
        req.on('error', reject);
        for (const item of data)
            req.write(item);
        req.end();
        function responseHandler(res) {
            const status = res.statusCode;
            let body = '';
            res
                .on('data', chunk => (body += chunk))
                .on('end', onEnd)
                .on('error', reject);
            function onEnd() {
                if (status < 200 || status >= 300) {
                    return reject(new Error(`${status} ${body}`));
                }
                const json = json_1.tryParseJSON(body);
                if (!json)
                    return reject(new Error(`Unkown Error: "${body}"`));
                if (!json.errno)
                    resolve(json);
                const msg = `${json.errno} ${json['errmsg'] || 'Unkown Error'}`;
                const err = new Error(msg);
                err['errno'] = json.errno;
                return reject(err);
            }
        }
    });
}
exports.request = request;
function optionsFromUrl(raw) {
    const url = new URL(raw);
    const ssl = url.protocol === 'https:';
    return {
        host: ssl || url.protocol === 'http:' ? url.hostname : 'localhost',
        port: url.port || (ssl ? 443 : 80),
        path: url.pathname + (url.search ? url.search : ''),
        method: 'GET'
    };
}
exports.optionsFromUrl = optionsFromUrl;
